@@ -33,10 +33,12 @@ import statistics
 from sklearn.metrics import f1_score
 
 
-def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=None, list_kpi=[], n_jobs=None, metric=None, num_rounds=None,early_stop=None,log=None,random_state=42):
-    folds = 5
+def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=None,neg_bagging_fraction=False,folds=None, list_kpi=[], n_jobs=None, metric=None, num_rounds=None,early_stop=None,log=None,random_state=42):
     
-    kf = StratifiedKFold(n_splits=folds, shuffle=True, random_state=random_state)
+    
+    if folds is None:
+        n_s = 5    
+        folds = StratifiedKFold(n_splits=n_s, shuffle=True, random_state=random_state)
     ''' 
     test_size = hcm.get_test_size(X,log)
     if test_size == 0.20:
@@ -49,13 +51,15 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
     X_train = X
     y_train = y    
     
-    X_train.reset_index(inplace=True,drop=True)
-    y_train.reset_index(inplace=True,drop=True)   
+    #X_train.reset_index(inplace=True,drop=True)
+    #y_train.reset_index(inplace=True,drop=True)   
  
     gc.collect()
    
     models = []
     
+    list_train_best_iteration = []
+    
     list_valid_score = [] 
     list_train_score = []
     
@@ -96,12 +100,13 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
     
     with_bias = False
     
-    for train_idx, valid_idx in kf.split(X_train, y_train):
-        train_data = X_train.iloc[train_idx,:], y_train[train_idx]
-        valid_data = X_train.iloc[valid_idx,:], y_train[valid_idx]
+    for train_idx, valid_idx in folds.split(X_train, y_train):
+        train_data = X_train.iloc[train_idx], y_train.iloc[train_idx]
+        valid_data = X_train.iloc[valid_idx], y_train.iloc[valid_idx]
 
         args = {
                 "trial":trial, "train":train_data, "val":valid_data, "fn_params":fn_params, "cat_features":None, 
+                "neg_bagging_fraction":neg_bagging_fraction, 
                 "metric":metric,"n_jobs":n_jobs,"num_rounds":num_rounds,"early_stop":early_stop,"log":log,"random_state":random_state
                 }
 
@@ -112,6 +117,8 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
         
         
         
+        list_train_best_iteration.append(kpi["train/best_iteration"])
+        
         list_train_precision.append(kpi["train/precision"])
         list_train_recall.append(kpi["train/recall"])
         list_train_f1.append(kpi["train/f1"])
@@ -201,25 +208,9 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
     
     
     
-    ''' 
-    mean_TP_train_opt = statistics.mean(list_TP_train_opt)
-    std_TP_train_opt = statistics.pstdev(list_TP_train_opt)
-    mean_FP_train_opt = statistics.mean(list_FP_train_opt)
-    std_FP_train_opt = statistics.pstdev(list_FP_train_opt)
-    mean_TN_train_opt = statistics.mean(list_TN_train_opt)
-    std_TN_train_opt = statistics.pstdev(list_TN_train_opt)
-    mean_FN_train_opt = statistics.mean(list_FN_train_opt)
-    std_FN_train_opt = statistics.pstdev(list_FN_train_opt)
+    mean_train_best_iteration = statistics.mean(list_train_best_iteration)
+    std_train_best_iteration = statistics.pstdev(list_train_best_iteration)
     
-    mean_TP_valid_opt = statistics.mean(list_TP_valid_opt)
-    std_TP_valid_opt = statistics.pstdev(list_TP_valid_opt)
-    mean_FP_valid_opt = statistics.mean(list_FP_valid_opt)
-    std_FP_valid_opt = statistics.pstdev(list_FP_valid_opt)
-    mean_TN_valid_opt = statistics.mean(list_TN_valid_opt)
-    std_TN_valid_opt = statistics.pstdev(list_TN_valid_opt)
-    mean_FN_valid_opt = statistics.mean(list_FN_valid_opt)
-    std_FN_valid_opt = statistics.pstdev(list_FN_valid_opt)
-    '''
     
 
     mean_train_precision = statistics.mean(list_train_precision)           
@@ -256,14 +247,18 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
     bias = abs(mean_train_score - mean_valid_score)
     
     result_iter = {
+        
         'trial_number':trial.number,
+        'mean_best_iteration':mean_train_best_iteration,
+        'std_best_iteration':std_train_best_iteration,   
         'bias': bias,
         'overfitting':with_bias,
         'mean_train_score':mean_train_score,
         'std_train_score':std_train_score,  
         
         'mean_valid_score':mean_valid_score,
-        'std_valid_score':std_valid_score,       
+        'std_valid_score':std_valid_score,           
+  
           
     }   
     
@@ -273,9 +268,10 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
         split_dic[key_split] = list_valid_score[0]
         result_iter.update(split_dic)
     else:
-        for i in range(folds):
+        for i in range(folds.get_n_splits()):
             split_train_dic = {}
             split_valid_dic = {}
+            split_best_iteration_dic = {}
             lb_fold = i+1
             
             key_split_train = "split_{}_train_score".format(lb_fold)
@@ -285,6 +281,10 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
             key_split_valid = "split_{}_valid_score".format(lb_fold)          
             split_valid_dic[key_split_valid] = list_valid_score[i] if with_bias==False else -1
             result_iter.update(split_valid_dic)
+            
+            key_split_best_iteration = "split_{}_best_iter".format(lb_fold)          
+            split_best_iteration_dic[key_split_best_iteration] = list_train_best_iteration[i] if with_bias==False else -1
+            result_iter.update(split_best_iteration_dic)
         
     
  
@@ -374,16 +374,17 @@ def lgb_f1_score(y_hat, data):
     return 'f1', f1_score(y_true, y_hat), True
 
 
-def fit_lgbm_with_pruning(trial=None, train=None, val=None, fn_params=None, cat_features=None,metric=None,n_jobs=1, num_rounds=None,early_stop=None,log=None,random_state=None ):
+def fit_lgbm_with_pruning(trial=None, train=None, val=None, fn_params=None, neg_bagging_fraction=False, cat_features=None,metric=None,n_jobs=1, num_rounds=None,early_stop=None,log=None,random_state=None ):
     """Train Light GBM model"""    
     X_train, y_train = train
     X_valid, y_valid = val
     
     train_params ,  dataset_params = fn_params(trial)
-        
-    nb_f = nbf_lgb_model.get_neg_bagging_fraction_params(y_train,{},log)
     
-    train_params.update(nb_f)
+    if neg_bagging_fraction:
+        
+        nb_f = nbf_lgb_model.get_neg_bagging_fraction_params(y_train,{},log)        
+        train_params.update(nb_f)
     
     params_config = {"seed": random_state, "n_jobs":n_jobs,"objective":'binary' , "metric": metric}
 
@@ -487,7 +488,7 @@ def fit_lgbm_with_pruning(trial=None, train=None, val=None, fn_params=None, cat_
            #'valid/FN_opt': FN_valid_opt,  
            
            #'valid/opt_threshold_valid': opt_threshold_valid, 
-           
+           'train/best_iteration':model.best_iteration,
            'bias':bias
           }
     return model, kpi
@@ -514,7 +515,7 @@ def perf_measure(y_actual, y_hat):
     return(TP, FP, TN, FN)
   
 
-def lgb_model_optuna(X_train=None, y_train=None,n_trials=2,fast_check=True,fn_params=None, 
+def lgb_model_optuna(X_train=None, y_train=None,n_trials=2,fast_check=True,fn_params=None, neg_bagging_fraction=False, folds=None,
                       n_jobs=None, metric=None, num_rounds=None,early_stop=None,log=None,random_state=42):
     
     if log == 0:
@@ -526,7 +527,8 @@ def lgb_model_optuna(X_train=None, y_train=None,n_trials=2,fast_check=True,fn_pa
     study = optuna.create_study(pruner=pruner,sampler=sampler,direction="maximize")
 
 
-    func = lambda trial: objective_with_prune(trial,fast_check=fast_check, X=X_train, y=y_train,fn_params=fn_params, list_kpi=list_kpi_,
+    func = lambda trial: objective_with_prune(trial,fast_check=fast_check, X=X_train, y=y_train,fn_params=fn_params,
+                                              neg_bagging_fraction=neg_bagging_fraction,folds=folds, list_kpi=list_kpi_,
                                               n_jobs=n_jobs, metric=metric, num_rounds=num_rounds,early_stop=early_stop,log=log,
                                               random_state=random_state)
     
